@@ -19,6 +19,9 @@ from tox.util.path import ensure_empty_dir
 
 from .config import DepConfig
 
+#: maximum parsed shebang interpreter length (see: prepend_shebang_interpreter)
+MAXINTERP = 2048
+
 
 class CreationConfig:
     def __init__(
@@ -154,7 +157,7 @@ class VirtualEnv(object):
 
     @property
     def path(self):
-        """ Path to environment base dir. """
+        """Path to environment base dir."""
         return self.envconfig.envdir
 
     @property
@@ -163,7 +166,7 @@ class VirtualEnv(object):
 
     @property
     def name(self):
-        """ test environment name. """
+        """test environment name."""
         return self.envconfig.envname
 
     def __repr__(self):
@@ -430,6 +433,9 @@ class VirtualEnv(object):
                 redirect=reporter.verbosity() < reporter.Verbosity.DEBUG,
                 env=env,
             )
+        except KeyboardInterrupt:
+            self.status = "keyboardinterrupt"
+            raise
         finally:
             sys.stdout = old_stdout
 
@@ -493,7 +499,7 @@ class VirtualEnv(object):
             env = os.environ.copy()
 
         # in any case we honor per-testenv setenv configuration
-        env.update(self.envconfig.setenv)
+        env.update(self.envconfig.setenv.export())
 
         env["VIRTUAL_ENV"] = str(self.path)
         return env
@@ -524,7 +530,7 @@ class VirtualEnv(object):
                 # happens if the same environment is invoked twice
                 message = "commands[{}] | {}".format(
                     i,
-                    " ".join([pipes.quote(str(x)) for x in argv]),
+                    " ".join(pipes.quote(str(x)) for x in argv),
                 )
                 action.setactivity(name, message)
                 # check to see if we need to ignore the return code
@@ -603,9 +609,12 @@ class VirtualEnv(object):
     def setupenv(self):
         if self.envconfig._missing_subs:
             self.status = (
-                "unresolvable substitution(s): {}. "
+                "unresolvable substitution(s):\n    {}\n"
                 "Environment variables are missing or defined recursively.".format(
-                    ",".join(["'{}'".format(m) for m in self.envconfig._missing_subs]),
+                    "\n    ".join(
+                        "{}: '{}'".format(section_key, exc.name)
+                        for section_key, exc in sorted(self.envconfig._missing_subs.items())
+                    ),
                 )
             )
             return
@@ -631,6 +640,9 @@ class VirtualEnv(object):
                 status = e
                 if self.envconfig.config.option.skip_missing_interpreters == "true":
                     default_ret_code = 0
+            except KeyboardInterrupt:
+                self.status = "keyboardinterrupt"
+                raise
             if status:
                 str_status = str(status)
                 command_log = envlog.get_commandlog("setup")
@@ -663,7 +675,7 @@ def prepend_shebang_interpreter(args):
     #
     # When preparing virtual environments in a file container which has large
     # length, the system might not be able to invoke shebang scripts which
-    # define interpreters beyond system limits (e.x. Linux as a limit of 128;
+    # define interpreters beyond system limits (e.g. Linux has a limit of 128;
     # BINPRM_BUF_SIZE). This method can be used to check if the executable is
     # a script containing a shebang line. If so, extract the interpreter (and
     # possible optional argument) and prepend the values to the provided
@@ -673,8 +685,9 @@ def prepend_shebang_interpreter(args):
     try:
         with open(args[0], "rb") as f:
             if f.read(1) == b"#" and f.read(1) == b"!":
-                MAXINTERP = 2048
-                interp = f.readline(MAXINTERP).rstrip().decode("UTF-8")
+                interp = f.readline(MAXINTERP + 1).rstrip().decode("UTF-8")
+                if len(interp) > MAXINTERP:  # avoid a truncated interpreter
+                    return args
                 interp_args = interp.split(None, 1)[:2]
                 return interp_args + args
     except (UnicodeDecodeError, IOError):
@@ -695,6 +708,8 @@ def tox_testenv_create(venv, action):
         args.append("--always-copy")
     if not venv.envconfig.download:
         args.append("--no-download")
+    else:
+        args.append("--download")
     # add interpreter explicitly, to prevent using default (virtualenv.ini)
     args.extend(["--python", str(config_interpreter)])
 
