@@ -3,11 +3,13 @@ import sys
 
 import py
 import pytest
+from six import PY2
 
 import tox
 from tox.interpreters import NoInterpreterInfo
 from tox.session.commands.run.sequential import installpkg, runtestenv
 from tox.venv import (
+    MAXINTERP,
     CreationConfig,
     VirtualEnv,
     getdigest,
@@ -77,22 +79,24 @@ def test_create(mocksession, newconfig):
         our_sys_path = py.path.local(sys.executable).realpath()
         assert our_sys_path == py.path.local(args[0]).realpath()
         # assert Envconfig.toxworkdir in args
-        assert venv.getcommandpath("easy_install", cwd=py.path.local())
+        assert venv.getcommandpath("pip", cwd=py.path.local())
     interp = venv._getliveconfig().base_resolved_python_path
     assert interp == venv.envconfig.python_info.executable
     assert venv.path_config.check(exists=False)
 
 
-def test_create_KeyboardInterrupt(mocksession, newconfig, mocker):
+@pytest.mark.parametrize("patched_venv_methodname", ["_pcall", "update"])
+def test_create_KeyboardInterrupt(mocksession, newconfig, mocker, patched_venv_methodname):
     config = newconfig(
         [],
         """\
         [testenv:py123]
+        deps = pip >= 19.3.1
         """,
     )
     mocksession.new_config(config)
     venv = mocksession.getvenv("py123")
-    mocker.patch.object(venv, "_pcall", side_effect=KeyboardInterrupt)
+    mocker.patch.object(venv, patched_venv_methodname, side_effect=KeyboardInterrupt)
     with pytest.raises(KeyboardInterrupt):
         venv.setupenv()
 
@@ -109,10 +113,10 @@ def test_commandpath_venv_precedence(tmpdir, monkeypatch, mocksession, newconfig
     mocksession.new_config(config)
     venv = mocksession.getvenv("py123")
     envconfig = venv.envconfig
-    tmpdir.ensure("easy_install")
+    tmpdir.ensure("pip")
     monkeypatch.setenv("PATH", str(tmpdir), prepend=os.pathsep)
-    envconfig.envbindir.ensure("easy_install")
-    p = venv.getcommandpath("easy_install")
+    envconfig.envbindir.ensure("pip")
+    p = venv.getcommandpath("pip")
     assert py.path.local(p).relto(envconfig.envbindir), p
 
 
@@ -770,21 +774,36 @@ class TestVenvTest:
         assert "PYTHONPATH" not in pcalls[0].env
 
 
-def test_env_variables_added_to_pcall(tmpdir, mocksession, newconfig, monkeypatch):
+def test_env_variables_added_to_pcall(tmpdir, mocksession, newconfig, monkeypatch, tmp_path):
     monkeypatch.delenv("PYTHONPATH", raising=False)
     pkg = tmpdir.ensure("package.tar.gz")
     monkeypatch.setenv("X123", "123")
     monkeypatch.setenv("YY", "456")
+    env_path = tmp_path / ".env"
+    env_file_content = "ENV_FILE_VAR = file_value"
+    env_path.write_text(env_file_content.decode() if PY2 else env_file_content)
+
     config = newconfig(
         [],
-        """\
+        r"""
+        [base]
+        base_var = base_value
+
         [testenv:python]
         commands=python -V
         passenv = x123
         setenv =
             ENV_VAR = value
+            ESCAPED_VAR = \{value\}
+            ESCAPED_VAR2 = \\{value\\}
+            BASE_VAR = {[base]base_var}
             PYTHONPATH = value
-        """,
+            TTY_VAR = {tty:ON_VALUE:OFF_VALUE}
+            COLON = {:}
+            REUSED_FILE_VAR = reused {env:ENV_FILE_VAR}
+            file| %s
+        """
+        % env_path,
     )
     mocksession._clearmocks()
     mocksession.new_config(config)
@@ -799,10 +818,18 @@ def test_env_variables_added_to_pcall(tmpdir, mocksession, newconfig, monkeypatc
         assert env is not None
         assert "ENV_VAR" in env
         assert env["ENV_VAR"] == "value"
+        assert env["ESCAPED_VAR"] == "{value}"
+        assert env["ESCAPED_VAR2"] == r"\{value\}"
+        assert env["COLON"] == ";" if sys.platform == "win32" else ":"
+        assert env["TTY_VAR"] == "OFF_VALUE"
+        assert env["ENV_FILE_VAR"] == "file_value"
+        assert env["REUSED_FILE_VAR"] == "reused file_value"
+        assert env["BASE_VAR"] == "base_value"
         assert env["VIRTUAL_ENV"] == str(venv.path)
         assert env["X123"] == "123"
         assert "PYTHONPATH" in env
         assert env["PYTHONPATH"] == "value"
+
     # all env variables are passed for installation
     assert pcalls[0].env["YY"] == "456"
     assert "YY" not in pcalls[1].env
@@ -878,20 +905,35 @@ def test_run_custom_install_command(newmocksession):
         [],
         """\
         [testenv]
-        install_command=easy_install {opts} {packages}
+        install_command=cool-installer {opts} {packages}
         """,
     )
     venv = mocksession.getvenv("python")
     venv.just_created = True
     venv.envconfig.envdir.ensure(dir=1)
+    venv.envconfig.envbindir.ensure("cool-installer")
     with mocksession.newaction(venv.name, "hello") as action:
         venv.run_install_command(packages=["whatever"], action=action)
     pcalls = mocksession._pcalls
     assert len(pcalls) == 1
-    assert "easy_install" in pcalls[0].args[0]
+    assert "cool-installer" in pcalls[0].args[0]
     assert pcalls[0].args[1:] == ["whatever"]
 
 
+def test_run_install_command_handles_KeyboardInterrupt(newmocksession, mocker):
+    mocksession = newmocksession([], "")
+    venv = mocksession.getvenv("python")
+    venv.just_created = True
+    venv.envconfig.envdir.ensure(dir=1)
+
+    mocker.patch.object(venv, "_pcall", side_effect=KeyboardInterrupt)
+    with mocksession.newaction(venv.name, "hello") as action:
+        with pytest.raises(KeyboardInterrupt):
+            venv.run_install_command(packages=["whatever"], action=action)
+
+    assert venv.status == "keyboardinterrupt"
+
+
 def test_command_relative_issue36(newmocksession, tmpdir, monkeypatch):
     mocksession = newmocksession(
         [],
@@ -1108,6 +1150,18 @@ def test_tox_testenv_interpret_shebang_long_example(tmpdir):
     assert args == expected + base_args
 
 
+@pytest.mark.skipif("sys.platform == 'win32'", reason="no shebang on Windows")
+def test_tox_testenv_interpret_shebang_skip_truncated(tmpdir):
+    testfile = tmpdir.join("check_shebang_truncation.py")
+    original_args = [str(testfile), "arg1", "arg2", "arg3"]
+
+    # interpreter (too long example)
+    testfile.write("#!" + ("x" * (MAXINTERP + 1)))
+    args = prepend_shebang_interpreter(original_args)
+
+    assert args == original_args
+
+
 @pytest.mark.parametrize("download", [True, False, None])
 def test_create_download(mocksession, newconfig, download):
     config = newconfig(
